#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2022-04-27
#
# Monitor status of Juniper BGP Peers (IPv4 and IPv6)
#
# THX to Jeff Fern jeff[dash]cmk[at]fcse[dot]co[dot]uk for testing and providing feedback to make this addon happen
#
# 2022-04-30: code cleanup/streamlining
# 2022-05-09: added item to BgpPeer class, this is used in the discovery function
#

import copy
from typing import List, Dict, Optional, Tuple
from dataclasses import dataclass

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    startswith,
    OIDEnd,
    OIDBytes,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    StringByteTable,
)

from cmk.base.plugins.agent_based.utils.bgp_peer import (
    BgpPeer,
    bgp_get_peer_entry,
    bgp_render_ip_address,
)

from cmk.base.plugins.agent_based.utils.juniper_bgp_peer import (
    juniper_afi_safi_mapping,
)


@dataclass
class JuniperPrefixes:
    afi: str
    safi: str
    accepted_prefixes: int
    metric_count: List[Tuple[str, int]]
    metric_rate: List[Tuple[str, int]]


def parse_juniper_bgp_peer(string_table: List[StringByteTable]) -> Optional[Dict[str, BgpPeer]]:
    jnxBgpM2Peer, jnxBgpM2PrefixCountersEntry = string_table
    peer_table = {}
    peer_prefixes = {}

    for entry in jnxBgpM2PrefixCountersEntry:
        oid_end, in_prefixes, in_prefixes_accepted, in_prefixes_rejected, out_prefixes, in_prefixes_active = entry
        index, afi, safi = oid_end.split('.')  # peer index, afi, safi
        prefixes = JuniperPrefixes(
            afi=afi,
            safi=safi,
            accepted_prefixes=int(in_prefixes_accepted),
            metric_rate=[],
            metric_count=[
                ('in_prefixes', int(in_prefixes)),
                # ('in_prefixes_accepted', int(in_prefixes_accepted)),
                ('in_prefixes_rejected', int(in_prefixes_rejected)),
                ('out_prefixes', int(out_prefixes)),
                ('in_prefixes_active', int(in_prefixes_active)),
            ],
        )

        if peer_prefixes.get(index):
            peer_prefixes[index].append(prefixes)
        else:
            peer_prefixes[index] = [prefixes]

    for entry in jnxBgpM2Peer:
        # extract values and adjust format
        oid_end = entry[0]
        entry.pop(0)
        PeerRemoteAddrType = entry[0]
        entry.pop(0)
        PeerIndex = entry[0]
        entry.pop(0)
        PeerRoutingInstance = entry[0]
        entry.pop(0)

        remote_address = bgp_render_ip_address(PeerRemoteAddrType, entry[0])
        entry[0] = remote_address

        bgp_peer = bgp_get_peer_entry(entry)

        prefixes = peer_prefixes.get(PeerIndex, [])
        for address_family in prefixes:
            address_family_str = juniper_afi_safi_mapping(address_family.afi, address_family.safi)
            item = f'{remote_address} {address_family_str}'
            # ToDo: add vrf name
            routing_instance = ''
            if PeerRoutingInstance != '0':
                routing_instance = f'VR {PeerRoutingInstance}'
                item = f'{item} {routing_instance}'
            peer_table[item] = copy.deepcopy(bgp_peer[remote_address])
            peer_table[item].accepted_prefixes = address_family.accepted_prefixes
            peer_table[item].metric_count += address_family.metric_count
            peer_table[item].metric_rate += address_family.metric_rate
            peer_table[item].item['address_family'] = address_family_str
            peer_table[item].item['routing_instance'] = routing_instance
    return peer_table


register.snmp_section(
    name='juniper_bgp_peer',
    parse_function=parse_juniper_bgp_peer,
    parsed_section_name='bgp_peer',
    supersedes=['bgp_peer'],
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2636.5.1.1.2',  # BGP4-V2-MIB-JUNIPER::jnxBgpM2Peer
            oids=[
                OIDEnd(),
                '1.1.1.10',  # jnxBgpM2PeerRemoteAddrType
                '1.1.1.14',  # jnxBgpM2PeerIndex
                '1.1.1.15',  # jnxBgpM2PeerRoutingInstance
                # needs to be in this order to match bgp_peer
                OIDBytes('1.1.1.11'),  # jnxBgpM2PeerRemoteAddr
                '1.1.1.2',  # jnxBgpM2PeerState
                '1.1.1.3',  # jnxBgpM2PeerStatus (admin status(?))
                '6.1.1.1',  # jnxBgpM2PeerInUpdates
                '6.1.1.2',  # jnxBgpM2PeerOutUpdates
                '6.1.1.3',  # jnxBgpM2PeerInTotalMessages
                '6.1.1.4',  # jnxBgpM2PeerOutTotalMessages
                '6.1.1.5',  # jnxBgpM2PeerFsmEstablishedTrans
                '4.1.1.1',  # jnxBgpM2PeerFsmEstablishedTime
                '4.1.1.2',  # jnxBgpM2PeerInUpdatesElapsedTime
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2636.5.1.1.2.6.2.1',  # BGP4-V2-MIB-JUNIPER::jnxBgpM2PrefixCountersEntry
            oids=[
                OIDEnd(),  # index.afi.safi
                # '1',  # jnxBgpM2PrefixCountersAfi
                # '2',  # jnxBgpM2PrefixCountersSafi
                '7',  # jnxBgpM2PrefixInPrefixes
                '8',  # jnxBgpM2PrefixInPrefixesAccepted
                '9',  # jnxBgpM2PrefixInPrefixesRejected
                '10',  # jnxBgpM2PrefixOutPrefixes
                '11',  # jnxBgpM2PrefixInPrefixesActive
            ]
        ),
    ],
    detect=startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2636.1.1.1'),
),

# jnxBgpM2CfgPeerAdminStatus
# 1.3.6.1.4.1.2636.5.1.1.2.8.1.1.1
# stop(1)
# start(2)
